//primitives.cc: Implementation file for useful primitives
//logic, latches

#include "signal.h"
#include "primitives.h"

//Gates: no memory, simple functions of the inputs
/////////////////////////////////////////////////////////////////

//1 input gates
signal_type INV(signal_type input)
{
	signal_type output;
	switch(input)
		{
		case True:
			output = False;
			break;
		case False:
			output = True;
			break;
		default:
			output = Unknown;
		} //switch
	return output;
} //INV


signal_type BUF(signal_type input)
{
	signal_type output;
	switch(input)
		{
		case True:
			output = True;
			break;
		case False:
			output = False;
			break;
		default:
			output = Unknown;
		} //switch
	return output;
} //BUF


//2 input gates (fast implementation)
signal_type OR2(signal_type inp0, signal_type inp1)
{
	//implemented as a CMOS gate (not wired-OR)
	signal_type output;
	switch(inp0)
		{
		case True:
			output = True;
			break;
		case False:
			switch(inp1)
				{
				case True:
					output = True;
				break;
				case False:
					output = False;
				break;
				default:
					output = Unknown;
				} //switch
			break;
		default:
			switch(inp1)
				{
				case True:
					output = True;
				break;
				default:
					output = Unknown;
				} //switch
		} //switch
	return output;
} //OR2


signal_type NOR2(signal_type inp0, signal_type inp1)
{
	return (INV(OR2(inp0, inp1)));
} //NOR2


signal_type AND2(signal_type inp0, signal_type inp1)
{
	signal_type output;
	switch(inp0)
		{
		case False:
			output = False;
			break;
		case True:
			switch(inp1)
				{
				case True:
					output = True;
				break;
				case False:
					output = False;
				break;
				default:
					output = Unknown;
				} //switch
			break;
		default:
			switch(inp1)
				{
				case False:
					output = False;
				break;
				default:
					output = Unknown;
				} //switch
		} //switch
	return output;
} //AND2


signal_type NAND2(signal_type inp0, signal_type inp1)
{
	return (INV(AND2(inp0, inp1)));
} //NAND2


//multiple input gates
signal_type ORN(signal_type* inputs, unsigned int number_of_inputs)
{
//handles an arbitrary number of inputs
//if any input is True, the output will be True
//if not, then if any input is Unknown, the output will be Unknown
//otherwise False
unsigned int index;
bool True_found = false;
bool Unknown_found = false;
signal_type output = Unknown;

for(index = 0; index < number_of_inputs; index++)
	{
	if(inputs[index] == True)
		{
			True_found = true;
		} //if
	else {
		if(inputs[index] != False) //not true and not false
			{
				Unknown_found = true;
			} //if
		} //else
	} //for 
if(True_found)
	{
		output = True;
	} //if
else
	{
		if(Unknown_found)
			{
				output = Unknown;
			} //if
		else
			output = False; //everything must be False
	} //else
	return output;
} //ORN


signal_type NORN(signal_type* inputs, unsigned int number_of_inputs)
{
	return(INV(ORN(inputs, number_of_inputs)));
} //NORN


signal_type ANDN(signal_type* inputs, unsigned int number_of_inputs)
{
//handles an arbitrary number of inputs
//if any input is False, the output will be False
//if not, then if any input is unknown, the output will be unknown
//otherwise True
unsigned int index;
bool False_found = false;
bool Unknown_found = false;
signal_type output = Unknown;

for(index = 0; index < number_of_inputs; index++)
	{
	if(inputs[index] == False)
		{
			False_found = true;
		} //if
	else {
		if(inputs[index] != True) //not true and not false
			{
				Unknown_found = true;
			} //if
		} //else
	} //for 
if(False_found)
	{
		output = False;
	} //if
else
	{
		if(Unknown_found)
			{
				output = Unknown;
			} //if
		else
			output = True; //everything must be True
	} //else
	return output;
} //ANDN


signal_type NANDN(signal_type* inputs, unsigned int number_of_inputs)
{
	return(INV(ANDN(inputs, number_of_inputs)));
} //NANDN


//Latches: simple memory elements
////////////////////////////////////////////////////////////////

latch::latch()
{
	Clear();
} //constructor


latch::~latch()
{
} //destructor


void latch::Update(signal_type clock, signal_type input)
{
	switch(clock)
		{
		case True:
			latch_state = input;
			break;
		case False:
			break;
		default:
			latch_state = Unknown;
		} //switch
} //Update


void latch::Clear()
{
	latch_state = Unknown;
} //Clear


signal_type latch::Get_Output()
{
	return latch_state;
} //Get_Output

///////////////////////////////////////////////////////////////
//End of implementation file

